@@ -2,6 +2,7 @@
 #################### OBJECT HELPERS ####################
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+
 #' Check if genes/features are present
 #'
 #' Check if genes are present in object and return vector of found genes.  Return warning messages for
@@ -9,6 +10,165 @@
 #'
 #' @param data Name of input data.  Currently only data of classes: Seurat, liger, data.frame,
 #' dgCMatrix, dgTMatrix, tibble are accepted.  Gene_IDs must be present in rownames of the data.
+#' @param features vector of features to check.
+#' @param case_check logical. Whether or not to check if features are found if the case is changed from the
+#' input list (Sentence case to Upper and vice versa).  Default is TRUE.
+#' @param case_check_msg logical. Whether to print message to console if alternate case features are found
+#' in addition to inclusion in returned list.  Default is TRUE.
+#' @param print_msg logical. Whether message should be printed if all features are found.  Default is TRUE.
+#' @param omit_warn logical. Whether to print message about features that are not found in current object.
+#'  Default is TRUE.
+#' @param return_none logical. Whether list of found vs. bad features should still be returned if no
+#' features are found.  Default is FALSE.
+#' @param seurat_assay Name of assay to pull feature names from if `data` is Seurat Object.
+#' Default is NULL which will check against features from all assays present.
+#'
+#' @import cli
+#' @importFrom purrr reduce
+#' @importFrom SeuratObject Features
+#' @importFrom stringr str_to_upper str_to_sentence
+#'
+#' @return A list of length 3 containing 1) found features, 2) not found features, 3) features found if
+#' case was modified.
+#'
+#' @export
+#'
+#' @concept check_util
+#'
+#' @examples
+#' \dontrun{
+#' features <- Feature_Present(data = obj_name, features = DEG_list, print_msg = TRUE,
+#' case_check = TRUE)
+#' found_features <- features[[1]]
+#' }
+#'
+
+Feature_Present <- function(
+    data,
+    features,
+    case_check = TRUE,
+    case_check_msg = TRUE,
+    print_msg = TRUE,
+    omit_warn = TRUE,
+    return_none = FALSE,
+    seurat_assay = NULL
+) {
+  # Check object type
+  # Seurat
+  accepted_types <- c("data.frame", "dgCMatrix", "dgTMatrix", "tibble")
+  if (inherits(x = data, what = "Seurat")) {
+    # set assay (if null set to active assay)
+    assays_present <- seurat_assay %||% Assays(object = data)
+
+    possible_features <- lapply(assays_present, function(j) {
+      Features(x = data, assay = j)
+    })
+
+    possible_features <- unlist(possible_features)
+  } else if ((class(x = data)[[1]] == "liger")) {
+    # get complete gene list
+    length_liger <- length(x = data@raw.data)
+
+    list_genes <- lapply(1:length_liger, function(x){
+      rownames(x = data@raw.data[[x]])
+    })
+
+    possible_features <- reduce(list_genes, function(x, y) {
+      union(x = x, y = y)})
+  } else if ((class(x = data) %in% accepted_types)) {
+    possible_features <- rownames(x = data)
+  } else {
+    all_accepted <- c(accepted_types, "Seurat", "liger")
+    cli_abort(message = c("Input data is currently accepted only in the following formats:",
+                          "i" = "{.field {glue_collapse_scCustom(input_string = all_accepted, and = FALSE)}}.")
+    )
+  }
+
+  # If any features not found
+  if (any(!features %in% possible_features)) {
+    bad_features <- features[!features %in% possible_features]
+    found_features <- features[features %in% possible_features]
+    if (length(x = found_features) == 0) {
+      if (isTRUE(x = return_none)) {
+        # Combine into list and return
+        feature_list <- list(
+          found_features = NULL,
+          bad_features = bad_features,
+          wrong_case_found_features = NULL
+        )
+        return(feature_list)
+      } else {
+        cli_abort(message ="No requested features found.")
+      }
+    }
+
+    # Return message of features not found
+    if (length(x = bad_features) > 0 && isTRUE(x = omit_warn)) {
+      cli_warn(message = c("The following features were omitted as they were not found:",
+                           "i" = "{.field {glue_collapse_scCustom(input_string = bad_features, and = TRUE)}}")
+      )
+    }
+
+    # Check if features found if case is changed.
+    if (isTRUE(x = case_check)) {
+      upper_bad_features <- str_to_upper(string = bad_features)
+      upper_found_features <- upper_bad_features[upper_bad_features %in% possible_features]
+
+      sentence_bad_features <- str_to_sentence(string = bad_features)
+      sentence_found_features <- sentence_bad_features[sentence_bad_features %in% possible_features]
+
+      # Combine case check
+      wrong_case_found_features <- c(upper_found_features, sentence_found_features)
+
+      # Additional messages if found.
+      if (length(x = wrong_case_found_features) > 0) {
+        if (isTRUE(x = case_check_msg)) {
+          cli_warn(message = c("NOTE: However, the following features were found: {.field {glue_collapse_scCustom(input_string = wrong_case_found_features, and = TRUE)}}",
+                               "i" = "Please check intended case of features provided.")
+          )
+        }
+        # Combine into list and return
+        feature_list <- list(
+          found_features = found_features,
+          bad_features = bad_features,
+          wrong_case_found_features = wrong_case_found_features
+        )
+        return(feature_list)
+      }
+    }
+    # Combine into list and return
+    feature_list <- list(
+      found_features = found_features,
+      bad_features = bad_features,
+      wrong_case_found_features = "NA (check not performed.  Set 'case_check = TRUE' to perform check."
+    )
+    return(feature_list)
+  }
+
+  # Print all found message if TRUE
+  if (isTRUE(x = print_msg)) {
+    cli_inform(message = "All features present.")
+  }
+
+  # Return full input gene list.
+  # Combine into list and return
+  feature_list <- list(
+    found_features = features,
+    bad_features = NULL,
+    wrong_case_found_features = NULL
+  )
+  return(feature_list)
+}
+
+
+
+#' Check if genes/features are present  `r lifecycle::badge("soft-deprecated")`
+#'
+#' Check if genes are present in object and return vector of found genes.  Return warning messages for
+#' genes not found.
+#'
+#' @param data Name of input data.  Currently only data of classes: Seurat, liger, data.frame,
+#' dgCMatrix, dgTMatrix, tibble are accepted.  Gene_IDs must be present in rownames of the data.
 #' @param gene_list vector of genes to check.
 #' @param case_check logical. Whether or not to check if features are found if the case is changed from the
 #' input list (Sentence case to Upper and vice versa).  Default is TRUE.
@@ -20,7 +180,7 @@
 #' @param return_none logical. Whether list of found vs. bad features should still be returned if no
 #' features are found.  Default is FALSE.
 #' @param seurat_assay Name of assay to pull feature names from if `data` is Seurat Object.
-#' Defaults to  `DefaultAssay(OBJ)` if NULL.
+#' Default is NULL which will check against features from all assays present.
 #'
 #' @import cli
 #' @importFrom purrr reduce
@@ -32,7 +192,7 @@
 #'
 #' @export
 #'
-#' @concept helper_util
+#' @concept check_util
 #'
 #' @examples
 #' \dontrun{
@@ -51,14 +211,24 @@ Gene_Present <- function(
   return_none = FALSE,
   seurat_assay = NULL
 ) {
+  lifecycle::deprecate_soft(when = "2.1.0",
+                            what = "Gene_Present()",
+                            with = "Feature_Present()",
+                            details = c("i" = "Please adjust code now to prepare for full deprecation.")
+  )
+
   # Check object type
   # Seurat
   accepted_types <- c("data.frame", "dgCMatrix", "dgTMatrix", "tibble")
   if (inherits(x = data, what = "Seurat")) {
     # set assay (if null set to active assay)
-    assay <- seurat_assay %||% DefaultAssay(object = data)
+    assays_present <- seurat_assay %||% Assays(object = data)
 
-    possible_features <- Features(x = data, assay = seurat_assay)
+    possible_features <- lapply(assays_present, function(j) {
+      Features(x = data, assay = j)
+    })
+
+    possible_features <- unlist(possible_features)
   } else if ((class(x = data)[[1]] == "liger")) {
     # get complete gene list
     length_liger <- length(x = data@raw.data)
@@ -175,7 +345,7 @@ Gene_Present <- function(
 #'
 #' @export
 #'
-#' @concept helper_util
+#' @concept check_util
 #'
 #' @examples
 #' \dontrun{
@@ -224,7 +394,8 @@ Case_Check <- function(
 #' Check if meta data columns are present in object and return vector of found columns
 #' Return warning messages for meta data columns not found.
 #'
-#' @param seurat_object object name.
+#' @param object Seurat or Liger object name.
+#' @param seurat_object `r lifecycle::badge("deprecated")` deprecated.  Please use `object` instead.
 #' @param meta_col_names vector of column names to check.
 #' @param print_msg logical. Whether message should be printed if all features are found.  Default is TRUE.
 #' @param omit_warn logical. Whether to print message about features that are not found in current object. Default is TRUE.
@@ -237,26 +408,40 @@ Case_Check <- function(
 #'
 #' @export
 #'
-#' @concept helper_util
+#' @concept check_util
 #'
 #' @examples
 #' \dontrun{
-#' meta_variables <- Meta_Present(seurat_object = obj_name, gene_list = DEG_list, print_msg = TRUE)
+#' meta_variables <- Meta_Present(object = obj_name, meta_col_names = "percent_mito", print_msg = TRUE)
 #' }
 #'
 
 Meta_Present <- function(
-  seurat_object,
+  object,
+  seurat_object = deprecated(),
   meta_col_names,
   print_msg = TRUE,
   omit_warn = TRUE,
   return_none = FALSE
 ) {
-  # Check Seurat
-  Is_Seurat(seurat_object = seurat_object)
+  # Check is slot is supplied
+  if (lifecycle::is_present(seurat_object)) {
+    lifecycle::deprecate_warn(when = "2.1.0",
+                              what = "Meta_Present(seurat_object)",
+                              with = "Meta_Present(object)",
+                              details = c("!" = "Please adjust code now to prepare for full deprecation in v2.2.0.")
+    )
 
-  # get all features
-  possible_features <- colnames(x = seurat_object@meta.data)
+  }
+
+  # Set possible variables based on object type
+  if (inherits(x = object, what = "Seurat")) {
+    possible_features <- colnames(x = object@meta.data)
+  }
+
+  if (inherits(x = object, what = "liger")) {
+    possible_features <- colnames(x = object@cell.data)
+  }
 
   # If any features not found
   if (any(!meta_col_names %in% possible_features)) {
@@ -316,7 +501,7 @@ Meta_Present <- function(
 #'
 #' @export
 #'
-#' @concept helper_util
+#' @concept check_util
 #'
 #' @examples
 #' \dontrun{
@@ -376,7 +561,7 @@ Meta_Numeric <- function(
 #'
 #' @export
 #'
-#' @concept helper_util
+#' @concept check_util
 #'
 #' @examples
 #' \dontrun{
@@ -459,69 +644,6 @@ Reduction_Loading_Present <- function(
 }
 
 
-#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-#################### DATA ACCESS ####################
-#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-
-
-#' Get meta data from object
-#'
-#' Quick function to properly pull meta.data from objects.
-#'
-#' @param object Object of class Seurat or liger.
-#'
-#' @importFrom methods slot
-#'
-#' @return A data.frame containing cell-level meta data
-#'
-#' @export
-#'
-#' @concept helper_util
-#'
-#' @rdname Fetch_Meta
-#'
-#' @examples
-#' library(Seurat)
-#' meta_data <- Fetch_Meta(object = pbmc_small)
-#' head(meta_data, 5)
-#'
-
-Fetch_Meta <- function(object) {
-  UseMethod(generic = 'Fetch_Meta')
-}
-
-
-#' @rdname Fetch_Meta
-#' @export
-#' @concept helper_util
-#' @method Fetch_Meta Seurat
-
-Fetch_Meta.Seurat <- function(
-  object
-) {
-  # Pull meta data
-  object_meta <- object_meta <- slot(object = object, name = "meta.data")
-
-  return(object_meta)
-}
-
-
-#' @rdname Fetch_Meta
-#' @export
-#' @concept helper_util
-#' @method Fetch_Meta liger
-
-Fetch_Meta.liger <- function(
-  object
-) {
-
-  # Pull meta data
-  object_meta <- object_meta <- slot(object = object, name = "cell.data")
-
-  return(object_meta)
-}
-
-
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 #################### MATRIX HELPERS ####################
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -550,7 +672,7 @@ Fetch_Meta.liger <- function(
 #'
 #' @export
 #'
-#' @concept helper_util
+#' @concept read_merge_util
 #'
 #' @examples
 #' \dontrun{
@@ -679,7 +801,7 @@ Merge_Sparse_Data_All <- function(
 #'
 #' @export
 #'
-#' @concept helper_util
+#' @concept read_merge_util
 #'
 #' @examples
 #' \dontrun{
@@ -727,7 +849,7 @@ Extract_Modality <- function(
 #'
 #' @export
 #'
-#' @concept helper_util
+#' @concept read_merge_util
 #'
 #' @examples
 #' \dontrun{
@@ -785,11 +907,11 @@ Merge_Sparse_Multimodal_All <- function(
 #' @import cli
 #' @importFrom methods slot
 #'
-#' @references Re-implementing `CheckMatrix` only for sparse matrices with modified warning messages.  Original function from SeuratObject \url{https://github.com/mojaveazure/seurat-object/blob/9c0eda946e162d8595696e5280a6ecda6284db39/R/utils.R#L625-L650} (License: MIT).
+#' @references Re-implementing `CheckMatrix` only for sparse matrices with modified warning messages.  Original function from SeuratObject \url{https://github.com/satijalab/seurat-object/blob/9c0eda946e162d8595696e5280a6ecda6284db39/R/utils.R#L625-L650} (License: MIT).
 #'
 #' @export
 #'
-#' @concept helper_util
+#' @concept check_util
 #'
 #' @examples
 #' \dontrun{
@@ -848,7 +970,7 @@ CheckMatrix_scCustom <- function(
 #'
 #' @return matrix or data.frame with new column names.
 #'
-#' @concept helper_util
+#' @concept barcode_util
 #'
 #' @examples
 #' \dontrun{
@@ -950,7 +1072,7 @@ Replace_Suffix <- function(
 #'
 #' @return matrix or data.frame with new column names.
 #'
-#' @concept helper_util
+#' @concept barcode_util
 #'
 #' @examples
 #' \dontrun{
@@ -1028,7 +1150,7 @@ Change_Delim_Suffix <- function(
 #'
 #' @return matrix or data.frame with new column names.
 #'
-#' @concept helper_util
+#' @concept barcode_util
 #'
 #' @examples
 #' \dontrun{
@@ -1104,7 +1226,7 @@ Change_Delim_Prefix <- function(
 #'
 #' @return matrix or data.frame with new column names.
 #'
-#' @concept helper_util
+#' @concept barcode_util
 #'
 #' @examples
 #' \dontrun{
@@ -1561,7 +1683,7 @@ Pull_Cluster_Annotation <- function(
 #'
 #' @export
 #'
-#' @concept object_util
+#' @concept marker_annotation_util
 #'
 #' @examples
 #' \dontrun{
@@ -1608,6 +1730,56 @@ Rename_Clusters <- function(
 }
 
 
+#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+#################### GENERAL HELPERS ####################
+#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+
+#' Split vector into list
+#'
+#' Splits vector into chunks of x sizes
+#'
+#' @param x vector to split
+#' @param chunk_size size of chunks for vector to be split into, default is 100.
+#' @param verbose logical, print details of vector and split, default is FALSE.
+#'
+#' @return list with vector of X length
+#'
+#' @import cli
+#'
+#' @export
+#'
+#' @references Base code from stackoverflow post:
+#' \url{https://stackoverflow.com/a/3321659/15568251}
+#'
+#' @concept misc_util
+#'
+#' @examples
+#' vector <- c("gene1", "gene2", "gene3", "gene4", "gene5", "gene6")
+#'
+#' vector_list <- Split_Vector(x = vector, chunk_size = 3)
+#'
+
+Split_Vector <- function(
+    x,
+    chunk_size = 100,
+    verbose = FALSE
+) {
+  vector_list <- split(x, ceiling(x = seq_along(x)/chunk_size))
+
+  # Report info
+  if (isTRUE(x = verbose)) {
+    cli_inform(message = c("Original vector length: ({.field {length(x = x)}}).",
+                           "Split into {.field {length(x = vector_list)}} vectors of {.field {chunk_size}} items." ))
+  }
+
+  # return list
+  return(vector_list)
+
+}
+
+
+
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 #################### PROJECT ORGANIZATION ####################
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1766,3 +1938,159 @@ Copy_From_GCP <- function(
   # Copy files
   system(paste0("gsutil -m cp -r ", gcp_bucket_path, " ", folder_file_path))
 }
+
+
+#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+#################### GENE NAME HELPERS ####################
+#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+
+#' Update HGNC Gene Symbols
+#'
+#' Update human gene symbols using data from HGNC.  This function will store cached data in package directory using (BiocFileCache).  Use of this function requires internet connection on first use (or if setting `update_symbol_data = TRUE`).  Subsequent use does not require connection and will pull from cached data.
+#'
+#' @param input_data Data source containing gene names.  Accepted formats are:
+#' \itemize{
+#'  \item \code{charcter vector}
+#'  \item \code{Seurat Objects}
+#'  \item \code{data.frame}: genes as rownames
+#'  \item \code{dgCMatrix/dgTMatrix}: genes as rownames
+#'  \item \code{tibble}: genes in first column
+#' }
+#' @param update_symbol_data logical, whether to update cached HGNC data, default is NULL.
+#' If `NULL` BiocFileCache will check and prompt for update if cache is stale.
+#' If `FALSE` the BiocFileCache stale check will be skipped and current cache will be used.
+#' If `TRUE` the BiocFileCache stale check will be skipped and HGNC data will be downloaded.
+#' @param case_check_as_warn logical, whether case checking of features should cause abort or
+#' only warn, default is FALSE (abort).  Set to TRUE if atypical names (i.e. old LOC naming) are
+#' present in input_data.
+#' @param verbose logical, whether to print results detailing numbers of symbols, found, updated,
+#' and not found; default is TRUE.
+#'
+#' @return data.frame containing columns: input_features, Approved_Symbol (already approved; output unchanged), Not_Found_Symbol (symbol not in HGNC; output unchanged), Updated_Symbol (new symbol from HGNC; output updated).
+#'
+#' @import cli
+#' @importFrom dplyr mutate filter select across left_join join_by
+#' @importFrom magrittr "%>%"
+#' @importFrom stats complete.cases
+#' @importFrom stringr str_to_upper str_replace_na str_c str_replace
+#' @importFrom tidyr drop_na everything
+#'
+#' @export
+#'
+#' @concept misc_util
+#'
+#' @examples
+#' \dontrun{
+#' new_names <- Updated_HGNC_Symbols(input_data = Seurat_Object)
+#' }
+#'
+
+Updated_HGNC_Symbols <- function(
+    input_data,
+    update_symbol_data = NULL,
+    case_check_as_warn = FALSE,
+    verbose = TRUE
+) {
+  # Check BiocFileCache installed
+  BiocFileCache_check <- is_installed(pkg = "BiocFileCache")
+  if (isFALSE(x = BiocFileCache_check)) {
+    cli_abort(message = c(
+      "Please install the {.val BiocFileCache} package to use {.code Updated_HGNC_Symbols}",
+      "i" = "This can be accomplished with the following commands: ",
+      "----------------------------------------",
+      "{.field `install.packages({symbol$dquote_left}BiocManager{symbol$dquote_right})`}",
+      "{.field `BiocManager::install({symbol$dquote_left}BiocFileCache{symbol$dquote_right})`}",
+      "----------------------------------------"
+    ))
+  }
+
+  # Check input data type
+  accepted_types <- c("data.frame", "dgCMatrix", "dgTMatrix")
+
+  if (inherits(x = input_data, what = "Seurat")) {
+    input_symbols <- Features(input_data)
+  }
+  if ((class(x = input_data) %in% accepted_types)) {
+    input_symbols <- rownames(x = input_data)
+  }
+  if (inherits(x = input_data, what = "tibble")) {
+    input_symbols <-   input_data[, 1]
+  }
+  if (inherits(x = input_data, what = "character")) {
+    input_symbols <- input_data
+  }
+
+  # Check for duplicates
+  num_duplicated <- length(x = unique(x = input_symbols[duplicated(x = input_symbols)]))
+
+  if (num_duplicated > 0) {
+    cli_abort(message = c("Input data contains duplicate gene symbols.",
+                          "i" = "Check input data and/or make unique."))
+  }
+
+  # Check input symbols have correct case
+  case_check <- str_to_upper(input_symbols)
+  case_check <- gsub(pattern = "(.*C[0-9XY]+)ORF(.+)", replacement = "\\1orf\\2", x = case_check)
+  # Currently two genes that are case anomalies so correcting them here
+  case_check <- gsub(pattern = "HSA-MIR-", replacement = "hsa-mir-", x = case_check)
+
+  if (isFALSE(x = identical(x = input_symbols, y = case_check)) && isFALSE(x = case_check_as_warn)) {
+    cli_abort("Uncovered potential errors in case/capitalization of input symbols.  Please check case is correct.")
+  }
+  if (isFALSE(x = identical(x = input_symbols, y = case_check)) && isTRUE(x = case_check_as_warn)) {
+    cli_warn(c("Uncovered potential errors in case/capitalization of input symbols.  This may cause errors in updating gene symbols.",
+               "i" = "Please check case is correct and re-run if errors are found."))
+  }
+
+  # Download and process HGNC dataset if not already cached
+  hgnc_data_path <- download_hgnc_data(update = update_symbol_data)
+
+  hgnc_long_data <- readRDS(hgnc_data_path)
+
+  input_features_df <- data.frame("input_features" = input_symbols)
+
+  symbols_not_approved <- input_symbols[!input_symbols %in% hgnc_long_data$symbol]
+  symbols_approved <- input_symbols[input_symbols %in% hgnc_long_data$symbol]
+
+  input_features_df_approved <- input_features_df %>%
+    mutate("Approved_Symbol" = ifelse(.data[["input_features"]] %in% symbols_approved, .data[["input_features"]], NA)) %>%
+    drop_na()
+
+
+  input_features_updated_df <- hgnc_long_data %>%
+    filter(.data[["prev_symbol"]] %in% symbols_not_approved) %>%
+    mutate("Updated_Symbol" = symbol) %>%
+    select(any_of(c("prev_symbol", "Updated_Symbol"))) %>%
+    rename("input_features" = any_of("prev_symbol")) %>%
+    drop_na()
+
+  symbols_not_found <- data.frame("input_features" = symbols_not_approved[!symbols_not_approved %in% input_features_updated_df$input_features]) %>%
+    mutate("Not_Found_Symbol" = .data[["input_features"]])
+
+  merged_df <- left_join(input_features_df, y = input_features_df_approved, by = join_by("input_features")) %>%
+    left_join(symbols_not_found, by = join_by("input_features")) %>%
+    left_join(input_features_updated_df, by = join_by("input_features")) %>%
+    mutate(across(everything(), ~str_replace_na(string = .x, replacement = ""))) %>%
+    mutate(Output_Features = str_c(.data[["Approved_Symbol"]], .data[["Not_Found_Symbol"]], .data[["Updated_Symbol"]])) %>%
+    mutate(across(everything(), ~str_replace(string = .x, pattern = "^$", replacement = NA_character_))) %>%
+    filter(!(.data[["input_features"]] == "QARS" & .data[["Updated_Symbol"]] == "EPRS1"))
+
+  # Report the results
+  if (isTRUE(x = verbose)) {
+    num_features <- length(input_symbols)
+
+    num_updated <- sum(complete.cases(merged_df$Updated_Symbol))
+    num_not_found <- sum(complete.cases(merged_df$Not_Found_Symbol))
+    num_approved <- sum(complete.cases(merged_df$Approved_Symbol))
+
+    cli_inform(message = c("Input features contained {.field {format(x = num_features, big.mark = ',')}} gene symbols",
+                           "{col_green({symbol$tick})} {.field {format(x = num_approved, big.mark = ',')}} were already approved symbols.",
+                           "{col_blue({symbol$arrow_right})} {.field {format(x = num_updated, big.mark = ',')}} were updated to approved symbol.",
+                           "{col_red({symbol$cross})} {.field {format(x = num_not_found, big.mark = ',')}} were not found in HGNC dataset and remain unchanged."))
+  }
+
+  # Return results
+  return(merged_df)
+
+}
